@@ -5,6 +5,7 @@ use std::collections::{BinaryHeap, HashMap};
 use std::sync::Mutex;
 use std::time::{Duration, Instant};
 
+use crate::structs::WsResponse;
 use crate::{structs, tools, Value};
 use actix_web::{web, Error, HttpRequest, HttpResponse};
 use actix_web_actors::ws;
@@ -42,8 +43,8 @@ impl actix::Message for ListRooms {
 #[derive(Message)]
 #[rtype(result = "()")]
 pub struct Join {
-    pub id: usize,
-    pub name: String,
+    pub(crate) id: usize,
+    pub(crate) name: String,
     pub(crate) times: i32,
     pub(crate) minutes: u64,
 }
@@ -74,16 +75,38 @@ impl WsServer {
 }
 
 impl WsServer {
-    fn send_message(&self, room: &str, message: &str, skip_id: usize) {
+    fn send_join_message(&self, room: &str, websocket_response: WsResponse) {
+        if let Some((_, _, session_vec)) = self.rooms.get(room) {
+            let result_string = serde_json::to_string(&websocket_response).unwrap();
+
+            for id in session_vec {
+                if let Some((address, _)) = self.sessions.get(id) {
+                    let _ = address.do_send(Message(result_string.clone()));
+                }
+            }
+        }
+    }
+
+    fn send_message(&self, room: &str, mut websocket_response: WsResponse, skip_id: usize) {
         let mut locked_map = self.map.lock().unwrap();
-        if locked_map.contains_key(room) {
+        if let Some(v) = locked_map.get(room) {
+            //key 没有被消耗光
+            if let Some((addr, _)) = self.sessions.get(&skip_id) {
+                websocket_response.result = Some("remaining is not zero".to_string());
+                websocket_response.remaining = Some(v.times);
+                let _ = addr.do_send(Message(serde_json::to_string(&websocket_response).unwrap()));
+            }
             return;
+        } else {
+            websocket_response.result = Some("".to_string());
         }
         if let Some((mut times, minutes, sessions)) = self.rooms.get(room) {
+            let mut address_vec = vec![];
             for id in sessions {
                 if *id != skip_id {
-                    if let Some((addr, _)) = self.sessions.get(id) {
-                        let _ = addr.do_send(Message(message.to_owned()));
+                    if let Some((address, _)) = self.sessions.get(id) {
+                        // let _ = addr.do_send(Message(message.to_owned()));
+                        address_vec.push(address);
                         times -= 1;
                         if times == 0 {
                             break;
@@ -91,11 +114,17 @@ impl WsServer {
                     }
                 }
             }
+            websocket_response.remaining = Some(times);
+            let result_string = serde_json::to_string(&websocket_response).unwrap();
+            for i in address_vec {
+                let _ = i.do_send(Message(result_string.clone()));
+            }
+
             if 0 != times {
                 let create_time = tools::now_timestamps();
-                let mut v = Value::new(message, create_time);
-                v.times = times;
-                locked_map.insert(String::from(room), v);
+                let mut val = Value::new(websocket_response.message.unwrap(), create_time);
+                val.times = times;
+                locked_map.insert(String::from(room), val);
                 let delete_struct = structs::StructInDeleteQueue::new(
                     create_time + 60 * minutes,
                     create_time,
@@ -104,6 +133,12 @@ impl WsServer {
                 let mut locked_queue = self.queue.lock().unwrap();
                 locked_queue.push(delete_struct);
             }
+            if let Some((address, _)) = self.sessions.get(&skip_id) {
+                websocket_response.message = None;
+                websocket_response.result = Some("ok".to_string());
+                let _ =
+                    address.do_send(Message(serde_json::to_string(&websocket_response).unwrap()));
+            }
         }
     }
 }
@@ -126,14 +161,26 @@ impl Handler<Disconnect> for WsServer {
 
     fn handle(&mut self, msg: Disconnect, _: &mut Context<Self>) {
         if let Some((_, room)) = self.sessions.remove(&msg.id) {
-            let mut vec_len = 9999;
-            if let Some(tup) = self.rooms.get_mut(&room) {
+            let vec_len = if let Some(tup) = self.rooms.get_mut(&room) {
                 let index = (*tup).2.iter().position(|x| *x == msg.id).unwrap();
                 tup.2.remove(index);
-                vec_len = tup.2.len();
-            }
+                tup.2.len()
+            } else {
+                1
+            };
             if 0 == vec_len {
                 self.rooms.remove(&room);
+            } else {
+                let res = WsResponse {
+                    times: None,
+                    minutes: None,
+                    message: None,
+                    remaining: None,
+                    total: Some(vec_len),
+                    result: None,
+                };
+
+                self.send_join_message(&room, res);
             }
         }
     }
@@ -143,7 +190,18 @@ impl Handler<ClientMessage> for WsServer {
     type Result = ();
 
     fn handle(&mut self, msg: ClientMessage, _: &mut Context<Self>) {
-        self.send_message(&msg.room, msg.msg.as_str(), msg.id);
+        self.send_message(
+            &msg.room,
+            WsResponse {
+                times: None,
+                minutes: None,
+                message: Some(msg.msg),
+                remaining: None,
+                total: None,
+                result: None,
+            },
+            msg.id,
+        );
     }
 }
 
@@ -157,13 +215,23 @@ impl Handler<Join> for WsServer {
             times,
             minutes,
         } = msg;
-        self.rooms
+        let cur_room = self
+            .rooms
             .entry(name.clone())
-            .or_insert((times, minutes, Vec::new()))
-            .2
-            .push(id);
+            .or_insert((times, minutes, Vec::new()));
+        cur_room.2.push(id);
         let (_, room) = self.sessions.get_mut(&id).unwrap();
-        *room = name;
+        *room = name.clone();
+        let res = WsResponse {
+            times: Some((*cur_room).0),
+            minutes: Some((*cur_room).1),
+            message: None,
+            remaining: None,
+            total: Some(cur_room.2.len()),
+            result: None,
+        };
+
+        self.send_join_message(&name, res);
     }
 }
 const HEARTBEAT_INTERVAL: Duration = Duration::from_secs(5);
@@ -178,7 +246,7 @@ pub(crate) async fn chat_route(
         WsChatSession {
             id: 0,
             hb: Instant::now(),
-            room: "Main".to_owned(),
+            room: "".to_owned(),
             name: None,
             address: srv.get_ref().clone(),
         },
@@ -217,7 +285,7 @@ impl Actor for WsChatSession {
     }
 
     fn stopping(&mut self, _: &mut Self::Context) -> Running {
-        self.address.do_send(Disconnect { id: self.id });
+        let _ = self.address.do_send(Disconnect { id: self.id });
         Running::Stop
     }
 }
@@ -249,7 +317,6 @@ impl StreamHandler<Result<ws::Message, ws::ProtocolError>> for WsChatSession {
             }
             ws::Message::Text(text) => {
                 let m = text.trim();
-                // we check for /sss type of messages
                 if m.starts_with("/join") {
                     let v: Vec<&str> = m.splitn(3, ' ').collect();
 
@@ -262,7 +329,7 @@ impl StreamHandler<Result<ws::Message, ws::ProtocolError>> for WsChatSession {
                                 web::Query::<structs::Params>::from_query(&v[2][1..])
                             {
                                 if let Some(t) = params.times {
-                                    times = t;
+                                    times = times.max(t);
                                 }
                                 if let Some(mut t) = params.minutes {
                                     t = t.min(60 * 24 * 7);
@@ -270,7 +337,7 @@ impl StreamHandler<Result<ws::Message, ws::ProtocolError>> for WsChatSession {
                                 }
                             }
                         }
-                        self.address.do_send(Join {
+                        let _ = self.address.do_send(Join {
                             id: self.id,
                             name: self.room.clone(),
                             times,
@@ -310,8 +377,11 @@ impl WsChatSession {
     fn hb(&self, ctx: &mut ws::WebsocketContext<Self>) {
         ctx.run_interval(HEARTBEAT_INTERVAL, |act, ctx| {
             if Instant::now().duration_since(act.hb) > CLIENT_TIMEOUT {
-                println!("Websocket Client heartbeat failed, disconnecting!");
-                act.address.do_send(Disconnect { id: act.id });
+                println!(
+                    "Websocket Client heartbeat failed, disconnecting!id = {}.",
+                    act.id
+                );
+                let _ = act.address.do_send(Disconnect { id: act.id });
                 ctx.stop();
                 return;
             }
